/******************************************************************************
 *                                                                            *
 *                                                                            *
 *                                Copyright by                                *
 *                                                                            *
 *                              Azoteq (Pty) Ltd                              *
 *                          Republic of South Africa                          *
 *                                                                            *
 *                           Tel: +27(0)21 863 0033                           *
 *                           E-mail: info@azoteq.com                          *
 *                                                                            *
 * ========================================================================== *
 * @file        IQS324.h                                                    *
 * @brief       This file contains the header information for an IQS324     *
 *              Arduino library. The goal of the library is to provide easy   *
 *              functionality for initializing and using the Azoteq IQS324  *
 *              Hall-sensing device.                                          *
 * @author      Azoteq PTY Ltd                                                *
 * @version     v1.0                                                          *
 * @date        2025                                                          *
 * ========================================================================== *
 * @attention  Makes use of the following standard Arduino libraries:         *
 * - Arduino.h   -> included in IQS324.h, comes standard with Arduino       *
 * - Wire.h      -> Included in IQS324.h, comes standard with Arduino       *
 ******************************************************************************/

#ifndef IQS324_h
#define IQS324_h

// Include Files
#include "Arduino.h"
#include "Wire.h"
#include "IQS324_init.h"
#include "./inc/IQS324_addresses.h"

// Public Global Definitions
/* For use with Wire.h library. True argument with some functions closes the
I2C communication window. */
#define STOP true
/* For use with Wire.h library. False argument with some functions keeps the
I2C communication window open. */
#define RESTART false
#define IQS324_I2C_RETRY 10

// Device Info
#define IQS324_PRODUCT_NUM 2389

// Info Flags Byte Bits.
#define IQS324_SHOW_RESET_BIT 0
#define IQS324_POWER_MODE_BIT_0 0
#define IQS324_POWER_MODE_BIT_1 1

// System Settings Bits
#define IQS324_ACK_RESET_BIT 0
#define IQS324_SW_RESET_BIT 1
#define IQS324_FORCE_ATI_PF_BIT 2
#define IQS324_FORCE_ATI_HALL_BIT 3
#define IQS324_ZERO_NOW_BIT 4
#define IQS324_RESEED_PF 5
#define IQS324_EVENT_MODE_BIT 6

// Global Event Bits
#define IQS324_POWER_EVENT_BIT 0
#define IQS324_ATI_EVENT_BIT 1
#define IQS324_TOUCH_EVENT_BIT 2
#define IQS324_PROX_EVENT_BIT 3
#define IQS324_HALL_EVENT_BIT 4

// Touch Flag Bits
#define IQS324_BUTTON_ACTIVE_STATE_BIT 0
#define IQS324_MOVEMENT_ACTIVE_STATE_BIT 0

// Hall UI Flag Bits
#define IQS324_HALL_INTERVAL_CHANGED_BIT 0
#define IQS324_HALL_DIRECTION_BIT 1

/* Defines and structs for IQS324 states */
/**
 * @brief  IQS324 Init Enumeration.
 */
typedef enum
{
    IQS324_INIT_NONE,
    IQS324_INIT_VERIFY_PRODUCT,
    IQS324_INIT_READ_RESET,
    IQS324_INIT_CHIP_RESET,
    IQS324_INIT_UPDATE_SETTINGS,
    IQS324_INIT_CHECK_RESET,
    IQS324_INIT_ACK_RESET,
    IQS324_INIT_ATI,
    IQS324_INIT_WAIT_FOR_ATI,
    IQS324_INIT_ZERO,
    IQS324_INIT_READ_DATA,
    IQS324_INIT_ACTIVATE_EVENT_MODE,
    IQS324_INIT_DONE
} IQS324_init_e;

typedef enum
{
    IQS324_STATE_NONE,
    IQS324_STATE_START,
    IQS324_STATE_INIT,
    IQS324_STATE_CHECK_RESET,
    IQS324_STATE_WAIT_FORCE_COMMS,
    IQS324_STATE_RUN,
} IQS324_state_e;

typedef enum
{
    IQS324_HIGH_ACCURACY = 0,
    IQS324_NORMAL_POWER = 1,
    IQS324_LOW_POWER = 2,
    IQS324_ULP = 3
} IQS324_power_modes;

typedef struct
{
    IQS324_state_e state;
    IQS324_init_e init_state;
} IQS324_s;

/* IQS324 Memory map data variables, only save the data that might be used
during program runtime */
typedef struct
{
    uint8_t power_mode;          // 	0x2000
    uint8_t device_status;       // 	0x2001
    uint8_t hall_flags;          // 	0x2002
    uint8_t event_flags;         // 	0x2003
    uint8_t prox_states;         // 	0x2004
    uint8_t button_flags;        // 	0x2006
    uint8_t movement_flags;      // 	0x2007
    uint8_t hall_ui_interval[2]; // 	0x2064
} IQS324_MEMORY_MAP;

// Class Prototype
class IQS324
{
public:
    // Public Constructors
    IQS324();

    // Public Device States
    IQS324_s iqs324_state;

    // Public Variables
    IQS324_MEMORY_MAP IQSMemoryMap;
    bool new_data_available;
    uint16_t previous_interval;
    bool do_soft_reset;

    // Public Methods
    void begin(uint8_t deviceAddressIn, uint8_t readyPinIn, void (*interrupt_func)());
    bool init(void);
    void run(void);
    void queueValueUpdates(void);
    uint16_t getProductNum(bool stopOrRestart);
    uint8_t getmajorVersion(bool stopOrRestart);
    uint8_t getminorVersion(bool stopOrRestart);
    void acknowledgeReset(bool stopOrRestart);
    void ReATI(bool stopOrRestart);
    void SW_Reset(bool stopOrRestart);
    void writeMM(bool stopOrRestart);
    void clearRDY(void);
    bool getRDYStatus(void);
    void zeroNow(bool stopOrRestart);

    void setStreamMode(bool stopOrRestart);
    void setEventMode(bool stopOrRestart);

    void updateInfoFlags(bool stopOrRestart);
    IQS324_power_modes get_PowerMode(void);
    bool checkReset(void);

    bool button_buttonState(void);
    bool button_movementState(void);
    bool interval_changed(void);
    bool interval_direction(void);
    uint16_t interval_value(void);

    void force_I2C_communication(void);
    void ready_interrupt(void);

private:
    // Private Variables
    uint8_t _deviceAddress;
    uint8_t _readyPin;
    bool _deviceRDY;
    bool _force_comms_rdy_received;
    uint32_t force_comms_rdy_timestamp;

    // Private Methods
    void readRandomBytes(uint16_t memoryAddress, uint8_t numBytes, uint8_t bytesArray[], bool stopOrRestart);
    void writeRandomBytes(uint16_t memoryAddress, uint8_t numBytes, uint8_t bytesArray[], bool stopOrRestart);
    bool getBit(uint8_t data, uint8_t bit_number);
    uint8_t setBit(uint8_t data, uint8_t bit_number);
    uint8_t clearBit(uint8_t data, uint8_t bit_number);
};
#endif // IQS324_h
